﻿using System;
using System.ComponentModel;
using System.Data.Linq;
using System.Data.Linq.Mapping;
using System.Runtime.Serialization;

namespace InteLect.Mobile.Data
{
    [Database(Name = "InteLect")]
	public partial class InteLectDataContext : DataContext
	{
		private static MappingSource m_MappingSource = new AttributeMappingSource();
		
        #region Definiciones de métodos de extensibilidad

        partial void OnCreated();
        partial void InsertOrdenTrabajo(OrdenTrabajo instance);
        partial void UpdateOrdenTrabajo(OrdenTrabajo instance);
        partial void DeleteOrdenTrabajo(OrdenTrabajo instance);
        partial void InsertUsuario(Usuario instance);
        partial void UpdateUsuario(Usuario instance);
        partial void DeleteUsuario(Usuario instance);

        #endregion
		
		public InteLectDataContext() :
            base("Data Source=isostore:/InteLect.sdf", m_MappingSource)
		{
		}
		
		public Table<OrdenTrabajo> OrdenesTrabajo
		{
			get
			{
				return this.GetTable<OrdenTrabajo>();
			}
		}
		
		public Table<Usuario> Usuarios
		{
			get
			{
				return this.GetTable<Usuario>();
			}
		}
	}
	
	[Table(Name = "OrdenesTrabajo")]
	public partial class OrdenTrabajo : INotifyPropertyChanging, INotifyPropertyChanged
	{
		private static PropertyChangingEventArgs m_EmptyChangingEventArgs = new PropertyChangingEventArgs(String.Empty);
		
		private Guid m_OrdenTrabajoId;
		private Guid m_UsuarioId;
        private string m_Codigo;
		private DateTime m_Fecha;
		private Nullable<decimal> m_Cantidad;
        private string m_Compañia;
        private string m_Medida;
        private string m_Cliente;
        private string m_Direccion;
        private EntityRef<Usuario> m_Usuario;

        #region Definiciones de métodos de extensibilidad
        
        partial void OnLoaded();
        partial void OnValidate(ChangeAction action);
        partial void OnCreated();
        partial void OnOrdenTrabajoIdChanging(Guid value);
        partial void OnOrdenTrabajoIdChanged();
        partial void OnUsuarioIdChanging(Guid value);
        partial void OnUsuarioIdChanged();
        partial void OnCodigoChanging(string value);
        partial void OnCodigoChanged();
        partial void OnFechaChanging(DateTime value);
        partial void OnFechaChanged();
        partial void OnCantidadChanging(Nullable<decimal> value);
        partial void OnCantidadChanged();
        partial void OnCompañiaChanging(string value);
        partial void OnCompañiaChanged();
        partial void OnMedidaChanging(string value);
        partial void OnMedidaChanged();
        partial void OnClienteChanging(string value);
        partial void OnClienteChanged();
        partial void OnDireccionChanging(string value);
        partial void OnDireccionChanged();

        #endregion
		
		public OrdenTrabajo()
		{
			this.Initialize();
		}
		
		[Column(Name = "IdOrdenTrabajo", Storage = "m_OrdenTrabajoId", DbType = "UniqueIdentifier NOT NULL", IsPrimaryKey = true)]
		public Guid OrdenTrabajoId
		{
			get
			{
				return this.m_OrdenTrabajoId;
			}

			set
			{
				if ((this.m_OrdenTrabajoId != value))
				{
					this.OnOrdenTrabajoIdChanging(value);
					this.SendPropertyChanging();
					this.m_OrdenTrabajoId = value;
					this.SendPropertyChanged("OrdenTrabajoId");
					this.OnOrdenTrabajoIdChanged();
				}
			}
		}
		
		[Column(Name = "IdUsuario", Storage = "m_UsuarioId", DbType = "UniqueIdentifier NOT NULL")]
		public Guid UsuarioId
		{
			get
			{
				return this.m_UsuarioId;
			}

			set
			{
				if ((this.m_UsuarioId != value))
				{
					this.OnUsuarioIdChanging(value);
					this.SendPropertyChanging();
					this.m_UsuarioId = value;
					this.SendPropertyChanged("UsuarioId");
					this.OnUsuarioIdChanged();
				}
			}
		}

        [Column(Storage = "m_Codigo", DbType = "NVarChar(25) NOT NULL", CanBeNull = false)]
        public string Codigo
        {
            get
            {
                return this.m_Codigo;
            }

            set
            {
                if ((this.m_Codigo != value))
                {
                    this.OnCodigoChanging(value);
                    this.SendPropertyChanging();
                    this.m_Codigo = value;
                    this.SendPropertyChanged("Codigo");
                    this.OnCodigoChanged();
                }
            }
        }

		[Column(Storage = "m_Fecha", DbType = "DateTime NOT NULL")]
		public DateTime Fecha
		{
			get
			{
				return this.m_Fecha;
			}

			set
			{
				if ((this.m_Fecha != value))
				{
					this.OnFechaChanging(value);
					this.SendPropertyChanging();
					this.m_Fecha = value;
					this.SendPropertyChanged("Fecha");
					this.OnFechaChanged();
				}
			}
		}
		
		[Column(Storage = "m_Cantidad", DbType = "Decimal(10,4)")]
		public Nullable<decimal> Cantidad
		{
			get
			{
				return this.m_Cantidad;
			}

			set
			{
				if ((this.m_Cantidad != value))
				{
					this.OnCantidadChanging(value);
					this.SendPropertyChanging();
					this.m_Cantidad = value;
					this.SendPropertyChanged("Cantidad");
					this.OnCantidadChanged();
				}
			}
		}

        [Column(Storage = "m_Compañia", DbType = "NVarChar(100) NOT NULL", CanBeNull = false)]
        public string Compañia
        {
            get
            {
                return this.m_Compañia;
            }

            set
            {
                if ((this.m_Compañia != value))
                {
                    this.OnCompañiaChanging(value);
                    this.SendPropertyChanging();
                    this.m_Compañia = value;
                    this.SendPropertyChanged("Compañia");
                    this.OnCompañiaChanged();
                }
            }
        }

        [Column(Storage = "m_Medida", DbType = "NVarChar(100) NOT NULL", CanBeNull = false)]
        public string Medida
        {
            get
            {
                return this.m_Medida;
            }

            set
            {
                if ((this.m_Medida != value))
                {
                    this.OnMedidaChanging(value);
                    this.SendPropertyChanging();
                    this.m_Medida = value;
                    this.SendPropertyChanged("Medida");
                    this.OnMedidaChanged();
                }
            }
        }

        [Column(Storage = "m_Cliente", DbType = "NVarChar(100) NOT NULL", CanBeNull = false)]
        public string Cliente
        {
            get
            {
                return this.m_Cliente;
            }

            set
            {
                if ((this.m_Cliente != value))
                {
                    this.OnClienteChanging(value);
                    this.SendPropertyChanging();
                    this.m_Cliente = value;
                    this.SendPropertyChanged("Cliente");
                    this.OnClienteChanged();
                }
            }
        }

        [Column(Storage = "m_Direccion", DbType = "NVarChar(255) NOT NULL", CanBeNull = false)]
        public string Direccion
        {
            get
            {
                return this.m_Direccion;
            }

            set
            {
                if ((this.m_Direccion != value))
                {
                    this.OnDireccionChanging(value);
                    this.SendPropertyChanging();
                    this.m_Direccion = value;
                    this.SendPropertyChanged("Direccion");
                    this.OnDireccionChanged();
                }
            }
        }

		[Association(Name = "Usuario_OrdenTrabajo", Storage = "m_Usuario", ThisKey = "UsuarioId", OtherKey = "UsuarioId", IsForeignKey = true)]
		public Usuario Usuario
		{
			get
			{
				return this.m_Usuario.Entity;
			}

			set
			{
                Usuario previousValue = this.m_Usuario.Entity;
				if (((previousValue != value)
                            || (this.m_Usuario.HasLoadedOrAssignedValue == false)))
				{
					this.SendPropertyChanging();
					if ((previousValue != null))
					{
                        this.m_Usuario.Entity = null;
						previousValue.OrdenesTrabajo.Remove(this);
					}
                    this.m_Usuario.Entity = value;
					if ((value != null))
					{
						value.OrdenesTrabajo.Add(this);
						this.m_UsuarioId = value.UsuarioId;
					}
					else
					{
						this.m_UsuarioId = default(System.Guid);
					}
					this.SendPropertyChanged("Usuario");
				}
			}
		}
		
		public event PropertyChangingEventHandler PropertyChanging;
		
		public event PropertyChangedEventHandler PropertyChanged;
		
		protected virtual void SendPropertyChanging()
		{
			if ((this.PropertyChanging != null))
			{
				this.PropertyChanging(this, m_EmptyChangingEventArgs);
			}
		}
		
		protected virtual void SendPropertyChanged(String propertyName)
		{
			if ((this.PropertyChanged != null))
			{
				this.PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
			}
		}
		
		private void Initialize()
		{
			this.m_Usuario = default(EntityRef<Usuario>);
			OnCreated();
		}
		
		[OnDeserializing]
		public void OnDeserializing(StreamingContext context)
		{
			this.Initialize();
		}

        public static implicit operator InteLectInspeccionService.OrdenTrabajoMovil(OrdenTrabajo workOrder)
        {
            if (workOrder == null)
            {
                return null;
            }

            return new InteLectInspeccionService.OrdenTrabajoMovil()
            {
                OrdenTrabajoId = workOrder.OrdenTrabajoId,
                UsuarioId = workOrder.UsuarioId,
                Codigo = workOrder.Codigo,
                Fecha = workOrder.Fecha,
                Cantidad = workOrder.Cantidad,
                Compañia = workOrder.Compañia,
                Medida = workOrder.Medida,
                Cliente = workOrder.Cliente,
                Direccion = workOrder.Direccion
            };
        }

        public static implicit operator OrdenTrabajo(InteLectInspeccionService.OrdenTrabajoMovil workOrder)
        {
            if (workOrder == null)
            {
                return null;
            }

            return new OrdenTrabajo()
            {
                OrdenTrabajoId = workOrder.OrdenTrabajoId,
                UsuarioId = workOrder.UsuarioId,
                Codigo = workOrder.Codigo,
                Fecha = workOrder.Fecha,
                Cantidad = workOrder.Cantidad,
                Compañia = workOrder.Compañia,
                Medida = workOrder.Medida,
                Cliente = workOrder.Cliente,
                Direccion = workOrder.Direccion
            };
        }
    }
	
	[Table(Name = "Usuarios")]
	public partial class Usuario : INotifyPropertyChanging, INotifyPropertyChanged
	{
		private static PropertyChangingEventArgs m_EmptyChangingEventArgs = new PropertyChangingEventArgs(String.Empty);
		
		private Guid m_UsuarioId;
		private string m_Codigo;
		private string m_Nombre;
		private string m_Apellidos;
		private string m_NombreUsuario;
		private string m_Contraseña;
		private EntitySet<OrdenTrabajo> m_OrdenesTrabajo;
		private bool m_Serializing;
		
        #region Definiciones de métodos de extensibilidad

        partial void OnLoaded();
        partial void OnValidate(ChangeAction action);
        partial void OnCreated();
        partial void OnUsuarioIdChanging(Guid value);
        partial void OnUsuarioIdChanged();
        partial void OnCodigoChanging(string value);
        partial void OnCodigoChanged();
        partial void OnNombreChanging(string value);
        partial void OnNombreChanged();
        partial void OnApellidosChanging(string value);
        partial void OnApellidosChanged();
        partial void OnNombreUsuarioChanging(string value);
        partial void OnNombreUsuarioChanged();
        partial void OnContraseñaChanging(string value);
        partial void OnContraseñaChanged();

        #endregion
		
		public Usuario()
		{
			this.Initialize();
		}
		
		[Column(Name = "IdUsuario", Storage = "m_UsuarioId", DbType = "UniqueIdentifier NOT NULL", IsPrimaryKey = true)]
		public Guid UsuarioId
		{
			get
			{
				return this.m_UsuarioId;
			}

			set
			{
				if ((this.m_UsuarioId != value))
				{
					this.OnUsuarioIdChanging(value);
					this.SendPropertyChanging();
					this.m_UsuarioId = value;
					this.SendPropertyChanged("UsuarioId");
					this.OnUsuarioIdChanged();
				}
			}
		}
		
		[Column(Storage = "m_Codigo", DbType = "NVarChar(25) NOT NULL", CanBeNull = false)]
		public string Codigo
		{
			get
			{
				return this.m_Codigo;
			}

			set
			{
				if ((this.m_Codigo != value))
				{
					this.OnCodigoChanging(value);
					this.SendPropertyChanging();
					this.m_Codigo = value;
					this.SendPropertyChanged("Codigo");
					this.OnCodigoChanged();
				}
			}
		}
		
		[Column(Storage = "m_Nombre", DbType = "NVarChar(25) NOT NULL", CanBeNull = false)]
		public string Nombre
		{
			get
			{
				return this.m_Nombre;
			}

			set
			{
				if ((this.m_Nombre != value))
				{
					this.OnNombreChanging(value);
					this.SendPropertyChanging();
					this.m_Nombre = value;
					this.SendPropertyChanged("Nombre");
					this.OnNombreChanged();
				}
			}
		}
		
		[Column(Storage = "m_Apellidos", DbType = "NVarChar(50) NOT NULL", CanBeNull = false)]
		public string Apellidos
		{
			get
			{
				return this.m_Apellidos;
			}

			set
			{
				if ((this.m_Apellidos != value))
				{
					this.OnApellidosChanging(value);
					this.SendPropertyChanging();
					this.m_Apellidos = value;
					this.SendPropertyChanged("Apellidos");
					this.OnApellidosChanged();
				}
			}
		}
		
		[Column(Storage = "m_NombreUsuario", DbType = "NVarChar(25) NOT NULL", CanBeNull = false)]
		public string NombreUsuario
		{
			get
			{
				return this.m_NombreUsuario;
			}

			set
			{
				if ((this.m_NombreUsuario != value))
				{
					this.OnNombreUsuarioChanging(value);
					this.SendPropertyChanging();
					this.m_NombreUsuario = value;
					this.SendPropertyChanged("NombreUsuario");
					this.OnNombreUsuarioChanged();
				}
			}
		}
		
		[Column(Storage = "m_Contraseña", DbType = "NVarChar(25) NOT NULL", CanBeNull = false)]
		public string Contraseña
		{
			get
			{
				return this.m_Contraseña;
			}

			set
			{
				if ((this.m_Contraseña != value))
				{
					this.OnContraseñaChanging(value);
					this.SendPropertyChanging();
					this.m_Contraseña = value;
					this.SendPropertyChanged("Contraseña");
					this.OnContraseñaChanged();
				}
			}
		}
		
		[Association(Name = "Usuario_OrdenTrabajo", Storage = "m_OrdenesTrabajo", ThisKey = "UsuarioId", OtherKey = "UsuarioId")]
		public EntitySet<OrdenTrabajo> OrdenesTrabajo
		{
			get
			{
				if ((this.m_Serializing && (this.m_OrdenesTrabajo.HasLoadedOrAssignedValues == false)))
				{
					return null;
				}
				return this.m_OrdenesTrabajo;
			}

			set
			{
				this.m_OrdenesTrabajo.Assign(value);
			}
		}
		
		public event PropertyChangingEventHandler PropertyChanging;
		
		public event PropertyChangedEventHandler PropertyChanged;
		
		protected virtual void SendPropertyChanging()
		{
			if ((this.PropertyChanging != null))
			{
				this.PropertyChanging(this, m_EmptyChangingEventArgs);
			}
		}
		
		protected virtual void SendPropertyChanged(String propertyName)
		{
			if ((this.PropertyChanged != null))
			{
				this.PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
			}
		}
		
		private void AttachOrdenTrabajo(OrdenTrabajo entity)
		{
			this.SendPropertyChanging();
			entity.Usuario = this;
		}
		
		private void DetachOrdenTrabajo(OrdenTrabajo entity)
		{
			this.SendPropertyChanging();
			entity.Usuario = null;
		}
		
		private void Initialize()
		{
			this.m_OrdenesTrabajo = new EntitySet<OrdenTrabajo>(new Action<OrdenTrabajo>(this.AttachOrdenTrabajo), new Action<OrdenTrabajo>(this.DetachOrdenTrabajo));
			OnCreated();
		}
		
		[OnDeserializing]
		public void OnDeserializing(StreamingContext context)
		{
			this.Initialize();
		}
		
		[OnSerializing]
		public void OnSerializing(StreamingContext context)
		{
			this.m_Serializing = true;
		}
		
		[OnSerialized]
		public void OnSerialized(StreamingContext context)
		{
			this.m_Serializing = false;
		}

        public static implicit operator Usuario(InteLectInspeccionService.UsuarioMovil user)
        {
            if (user == null)
            {
                return null;
            }

            return new Usuario()
            {
                UsuarioId = user.UsuarioId,
                Codigo = user.Codigo,
                Nombre = user.Nombre,
                Apellidos = user.Apellidos,
                NombreUsuario = user.NombreUsuario,
                Contraseña = user.Contraseña
            };
        }

        public static implicit operator InteLectInspeccionService.UsuarioMovil(Usuario user)
        {
            if (user == null)
            {
                return null;
            }

            return new InteLectInspeccionService.UsuarioMovil()
            {
                UsuarioId = user.UsuarioId,
                Codigo = user.Codigo,
                Nombre = user.Nombre,
                Apellidos = user.Apellidos,
                NombreUsuario = user.NombreUsuario,
                Contraseña = user.Contraseña
            };
        }
    }
}
