﻿using System;
using System.Linq;
using System.Windows;
using InteLect.Mobile.Data;
using Microsoft.Phone.Controls;
using AuthenticateUserCompletedEventArgs = InteLect.Mobile.InteLectInspeccionService.AuthenticateUserCompletedEventArgs;
using InteLectInspeccionClient = InteLect.Mobile.InteLectInspeccionService.InteLectInspeccionClient;

namespace InteLect.Mobile
{
    public partial class LoginPage : PhoneApplicationPage
    {
        private InteLectInspeccionClient m_InspeccionClient = new InteLectInspeccionClient(); // Acceso al servicio web de inspección
        private InteLectDataContext m_DataContext = new InteLectDataContext(); // Acceso a la base de datos local
        private Usuario m_Usuario; // Usuario actual

        public LoginPage()
        {
            // Asignación de controladores de eventos para la página
            this.Loaded += new RoutedEventHandler(this.OnLoaded);

            // Asignación de controladores de eventos para el servicio web
            this.m_InspeccionClient.AuthenticateUserCompleted += new EventHandler<AuthenticateUserCompletedEventArgs>(this.OnAuthenticationCompleted);
            
            this.InitializeComponent();
        }

        private void OnLoaded(object sender, RoutedEventArgs e)
        {
            // Establecemos que no se ha iniciado sesión
            App.LoggedIn = false;

            try
            {
                // Si la base de datos local no existe la creamos
                if (!this.m_DataContext.DatabaseExists())
                {
                    this.m_DataContext.CreateDatabase();
                }
                else
                {
                    // Si existe tomamos el último usuario que utilizó la aplicación
                    this.m_Usuario = this.m_DataContext.Usuarios.FirstOrDefault();
                }
            }
            catch (Exception ex)
            {
                System.Diagnostics.Debug.WriteLine(ex);
                MessageBox.Show("Ha ocurrido un error al inicializar la aplicación. La aplicación se cerrará.", "Error", MessageBoxButton.OK);

                // Volvemos a la página principal
                this.NavigationService.GoBack();
            }

            // Si alguién utilizó previamente la aplicación establecemos el nombre de usuario
            if (this.m_Usuario != null)
            {
                this.tbxUsername.Text = this.m_Usuario.NombreUsuario;
            }
        }

        private void btnLogin_Click(object sender, RoutedEventArgs e)
        {
            // Deshabilitamos la página
            this.IsEnabled = false;

            // Llamámos al metodo del servicio web para la autenticación
            this.m_InspeccionClient.AuthenticateUserAsync(this.tbxUsername.Text, this.tbxPassword.Password);
        }

        private void OnAuthenticationCompleted(object sender, AuthenticateUserCompletedEventArgs e)
        {
            if (e.Error != null)
            {
                MessageBox.Show("Ha ocurrido un error al intentar iniciar sesión. Inténtalo de nuevo.", "Error", MessageBoxButton.OK);
            }
            else
            {
                var usuario = e.Result;

                // Si el usuario no es nulo quiere decir que la autenticación tubo éxito
                if (usuario != null)
                {
                    try
                    {
                        // Comprobamos si el último usuario que utilizó la aplicación es el mismo que acaba de iniciar sesión
                        if (!this.m_DataContext.Usuarios.Any(entity => entity.UsuarioId == usuario.UsuarioId))
                        {
                            // Si no lo es, recreamos la base de datos
                            this.m_DataContext.DeleteDatabase();
                            this.m_DataContext.CreateDatabase();

                            // Añadimos el usuario
                            this.m_DataContext.Usuarios.InsertOnSubmit(usuario);
                        }
                        else
                        {
                            // Como el usuario ya existe actualizamos los datos que vamos a usar
                            this.m_Usuario = this.m_DataContext.Usuarios.First();
                            this.m_Usuario.Codigo = usuario.Codigo;
                            this.m_Usuario.Nombre = usuario.Nombre;
                            this.m_Usuario.Apellidos = usuario.Apellidos;
                            this.m_Usuario.Contraseña = usuario.Contraseña;
                        }

                        // Guardamos los cambios realizados
                        this.m_DataContext.SubmitChanges();

                        // Establecemos que se ha iniciado sesión
                        App.LoggedIn = true;
                    }
                    catch (Exception ex)
                    {
                        System.Diagnostics.Debug.WriteLine(ex);
                        MessageBox.Show("Ha ocurrido un error al intentar acceder a la base de datos. La aplicación se cerrará.", "Error", MessageBoxButton.OK);
                    }

                    // Volvemos a la página principal
                    this.NavigationService.GoBack();
                }
                else
                {
                    MessageBox.Show("No se ha podido iniciar sesión con las credenciales proporcionadas. Comprueba los datos y vuelve a intentarlo.", "Sesión no iniciada", MessageBoxButton.OK);
                }
            }

            // Habilitamos la página
            this.IsEnabled = true;
        }
    }
}
