﻿using System;
using System.Collections.ObjectModel;
using System.Data.Linq;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Navigation;
using System.Windows.Threading;
using InteLect.Mobile.Data;
using Microsoft.Phone.Controls;
using Microsoft.Phone.Shell;
using DownloadWorkPlanCompletedEventArgs = InteLect.Mobile.InteLectInspeccionService.DownloadWorkPlanCompletedEventArgs;
using InteLectInspeccionClient = InteLect.Mobile.InteLectInspeccionService.InteLectInspeccionClient;
using UpdateWorkOrderCompletedEventArgs = InteLect.Mobile.InteLectInspeccionService.UpdateWorkOrderCompletedEventArgs;

namespace InteLect.Mobile
{
    public partial class MainPage : PhoneApplicationPage
    {
        private InteLectInspeccionClient m_InspeccionClient = new InteLectInspeccionClient(); // Acceso al servicio web de inspección
        private InteLectDataContext m_DataContext = new InteLectDataContext(); // Acceso a la base de datos local
        private Usuario m_Usuario; // Usuario actual
        private volatile int m_PendingUpdates; // Actualizaciones pendientes
        private ObservableCollection<OrdenTrabajo> m_OrdenesTrabajo = new ObservableCollection<OrdenTrabajo>(); // Plan de trabajo
        private DispatcherTimer m_Timer = new DispatcherTimer(); // Temporizador para la sincronización

        public MainPage()
        {
            // Asignación de controladores de eventos para la página
            this.Loaded += new RoutedEventHandler(this.OnLoaded);

            // Asignación de controladores de eventos para el servicio web
            this.m_InspeccionClient.DownloadWorkPlanCompleted += new EventHandler<DownloadWorkPlanCompletedEventArgs>(this.OnDownloadWorkPlanCompleted);
            this.m_InspeccionClient.UpdateWorkOrderCompleted += new EventHandler<UpdateWorkOrderCompletedEventArgs>(this.OnUpdateWorkOrderCompleted);

            this.InitializeComponent();

            // Asignamos el contexto de datos de la página para asociar a los controles visuales
            this.DataContext = this.m_OrdenesTrabajo;
        }

        private void OnLoaded(object sender, RoutedEventArgs e)
        {
            if (!App.LoggedIn.HasValue)
            {
                // Abrimos la página inicio de sesión si no se ha iniciado aún
                this.NavigationService.Navigate(new Uri("/LoginPage.xaml", UriKind.Relative));
            }
            else if (App.LoggedIn == false)
            {
                // Salimos de la aplicación (esto generará una excepción no controlada en desarrollo, continuando se saldrá de la aplicación)
                this.NavigationService.GoBack();
            }
            else
            {
                // Eliminación de controladores de eventos para la página (sólo queremos controlarlo una sola vez)
                this.Loaded -= new RoutedEventHandler(this.OnLoaded);

                // Establecemos el título de la página
                this.PageTitle.Text = "plan de trabajo";

                // Mostramos el menú
                this.ApplicationBar.IsVisible = true;

                // Establecemos el usuario actual
                this.m_Usuario = this.m_DataContext.Usuarios.First();

                // Comenzamos la descarga del plan de trabajo
                this.DownloadWorkPlan();

                // Configuramos y comenzamos el temporizador de sincronización (sincronizamos cada minuto)
                this.m_Timer.Interval = TimeSpan.FromMinutes(1);
                this.m_Timer.Tick += new EventHandler(this.OnTick);
                this.m_Timer.Start();
            }
        }

        private void OnTick(object sender, EventArgs e)
        {
            this.Synchronize();
        }

        private void btnUserProfile_Click(object sender, EventArgs e)
        {
            // Abrimos la página perfil de usuario
            this.NavigationService.Navigate(new Uri("/UserProfilePage.xaml", UriKind.Relative));
        }

        private void btnSynchronize_Click(object sender, EventArgs e)
        {
            this.Synchronize();
        }

        private void btnDownloadWorkPlan_Click(object sender, EventArgs e)
        {
            this.DownloadWorkPlan();
        }

        private void Synchronize()
        {
            // Desabilitamos el menú
            this.DisableMenu();

            // Deshabilitamos el temporizador
            this.m_Timer.Stop();

            // Obtenemos las órdenes de trabajo que se  han completado
            var workOrders = this.m_DataContext.OrdenesTrabajo.Where(orderTrabajo => orderTrabajo.Cantidad != null).ToArray();

            if (workOrders.Length > 0)
            {
                // Hay actualizaciones pendientes, así que las procesamos
                this.m_PendingUpdates = workOrders.Length;

                foreach (var workOrder in workOrders)
                {
                    this.m_InspeccionClient.UpdateWorkOrderAsync(this.m_Usuario, workOrder, workOrder);
                }
            }
            else
            {
                // No hay actualizaciones pendientes, así que habilitamos de nuevo el menú
                this.EnableMenu();

                // Habilitamos el temporizador
                this.m_Timer.Start();
            }
        }

        private void DownloadWorkPlan()
        {
            // Desabilitamos el menú
            this.DisableMenu();

            // Deshabilitamos el temporizador
            this.m_Timer.Stop();

            // Llamamos al método del servicio web para descargar el plan de trabajo
            this.m_InspeccionClient.DownloadWorkPlanAsync(this.m_DataContext.Usuarios.First());
        }

        private void OnUpdateWorkOrderCompleted(object sender, UpdateWorkOrderCompletedEventArgs e)
        {
            this.m_PendingUpdates--;

            if (e.Result)
            {
                // La actualización ha tenido éxito, así que ya podemos eliminar la órden de trabajo de la base de datos local
                var ordenTrabajo = e.UserState as OrdenTrabajo;
                this.m_OrdenesTrabajo.Remove(ordenTrabajo);
                this.m_DataContext.OrdenesTrabajo.DeleteOnSubmit(ordenTrabajo);
                this.m_DataContext.SubmitChanges();
            }

            if (this.m_PendingUpdates == 0)
            {
                // Habilitamos los botones menú
                this.EnableMenu();

                // Habilitamos el temporizador
                this.m_Timer.Start();
            }
        }

        private void OnDownloadWorkPlanCompleted(object sender, DownloadWorkPlanCompletedEventArgs e)
        {
            var workPlan = e.Result;

            // Comprobamos si hay órdenes de trabajo pendientes
            if (workPlan != null &&
                workPlan.Count > 0)
            {
                // Procesamos cada una de ellas
                foreach (var ordenTrabajo in e.Result)
                {
                    if (!this.m_DataContext.OrdenesTrabajo.Any(entity => entity.OrdenTrabajoId == ordenTrabajo.OrdenTrabajoId))
                    {
                        // Si no existe en la base de datos local la añadimos
                        this.m_DataContext.OrdenesTrabajo.InsertOnSubmit(ordenTrabajo);
                    }
                    else
                    {
                        // Si existe, la actualizamos
                        var workOrder = this.m_DataContext.OrdenesTrabajo.Single(entity => entity.OrdenTrabajoId == ordenTrabajo.OrdenTrabajoId);
                        workOrder.Codigo = ordenTrabajo.Codigo;
                        workOrder.Fecha = ordenTrabajo.Fecha;
                        workOrder.Compañia = ordenTrabajo.Compañia;
                        workOrder.Medida = ordenTrabajo.Medida;
                        workOrder.Cliente = ordenTrabajo.Cliente;
                        workOrder.Direccion = ordenTrabajo.Direccion;
                    }
                }

                // Guardamos los cambios en la base de datos local
                this.m_DataContext.SubmitChanges();
            }

            // Repoblamos la colección enlazada al control visual
            this.m_OrdenesTrabajo.Clear();
            this.m_DataContext.OrdenesTrabajo.Where(entity => entity.Cantidad == null).OrderBy(entity => entity.Fecha).ToList().ForEach(entity => this.m_OrdenesTrabajo.Add(entity));

            // Habilitamos los botones menú
            this.EnableMenu();

            // Habilitamos el temporizador
            this.m_Timer.Start();
        }

        private void EnableMenu()
        {
            // Habilitamos cada botón del menú
            foreach (ApplicationBarIconButton button in this.ApplicationBar.Buttons)
            {
                button.IsEnabled = true;
            }
        }

        private void DisableMenu()
        {
            // Deshabilitamos cada botón del menú
            foreach (ApplicationBarIconButton button in this.ApplicationBar.Buttons)
            {
                button.IsEnabled = false;
            }
        }

        private void OnOrdenTrabajoHold(object sender, Microsoft.Phone.Controls.GestureEventArgs e)
        {
            // Obtenemos la órden de trabajo seleccionada
            var ordenTrabajo = (sender as StackPanel).DataContext as OrdenTrabajo;

            // Abrimos la página para actualizar la orden de trabajo
            this.NavigationService.Navigate(new Uri(string.Concat("/WorkOrderPage.xaml?Id=",ordenTrabajo.OrdenTrabajoId) , UriKind.Relative));
        }

        private void OnWorkPlanFiltering(object sender, FilterEventArgs e)
        {
            // Filtramos las órdenes de trabajo en el control visual (sólo necesitamos las no procesadas)
            e.Accepted = false;
            var ordenTrabajo = e.Item as OrdenTrabajo;
            if (ordenTrabajo != null)
            {
                e.Accepted = (ordenTrabajo.Cantidad == null);
            }
        }

        protected override void OnNavigatedTo(NavigationEventArgs e)
        {
            // Actualizamos el plan de trabajo cada vez que se vuelva a la página principal
            var workPlan = this.Resources["WorkPlan"] as CollectionViewSource;
            if (workPlan != null &&
                workPlan.View != null)
            {
                // Esto forzará que se vuelva a filtrar el plan de trabajo para actualizar los controles visuales
                this.m_DataContext.Refresh(RefreshMode.OverwriteCurrentValues, this.m_OrdenesTrabajo);
                workPlan.View.Refresh();
            }

            // Habilitamos el temporizador
            this.m_Timer.Start();
        }

        protected override void OnNavigatedFrom(NavigationEventArgs e)
        {
            // Deshabilitamos el temporizador
            this.m_Timer.Stop();
        }
    }
}
