﻿using System;
using System.Linq;
using System.Windows;
using InteLect.Mobile.Data;
using Microsoft.Phone.Controls;
using ChangeUserPasswordCompletedEventArgs = InteLect.Mobile.InteLectInspeccionService.ChangeUserPasswordCompletedEventArgs;
using InteLectDataContext = InteLect.Mobile.Data.InteLectDataContext;
using InteLectInspeccionClient = InteLect.Mobile.InteLectInspeccionService.InteLectInspeccionClient;

namespace InteLect.Mobile
{
    public partial class UserProfilePage : PhoneApplicationPage
    {
        private InteLectInspeccionClient m_InspeccionClient = new InteLectInspeccionClient(); // Acceso al servicio web de inspección
        private InteLectDataContext m_DataContext = new InteLectDataContext(); // Acceso a la base de datos local
        private Usuario m_Usuario; // Usuario actual

        public UserProfilePage()
        {
            // Asignación de controladores de eventos para la página
            this.Loaded += new RoutedEventHandler(this.OnLoaded);

            // Asignación de controladores de eventos para el servicio web
            this.m_InspeccionClient.ChangeUserPasswordCompleted += new EventHandler<ChangeUserPasswordCompletedEventArgs>(this.OnChangeUserPasswordCompleted);
            
            this.InitializeComponent();
        }

        private void OnLoaded(object sender, RoutedEventArgs e)
        {
            // Asignamos el contexto de datos de la página para asociar a los controles visuales
            this.DataContext = (this.m_Usuario = this.m_DataContext.Usuarios.First());
        }

        private void OnChangeUserPasswordCompleted(object sender, ChangeUserPasswordCompletedEventArgs e)
        {
            if (e.Error != null)
            {
                MessageBox.Show("Ha ocurrido un error al intentar cambiar la contraseña. Inténtalo de nuevo.", "Error", MessageBoxButton.OK);
            }
            else
            {
                if (e.Result)
                {
                    // Como to fue bien, guardamos la nueva contraseña
                    this.m_Usuario.Contraseña = this.tbxPassword1.Password;
                    this.m_DataContext.SubmitChanges();
                    MessageBox.Show("La contraseña se ha cambiado correctamente.", "Contraseña cambiada", MessageBoxButton.OK);

                    // Volvemos a la página principal
                    this.NavigationService.GoBack();
                }
                else
                {
                    MessageBox.Show("No se ha podido cambiar la contraseña con las credenciales proporcionadas. Comprueba los datos y vuelve a intentarlo.", "Contraseña no cambiada", MessageBoxButton.OK);
                }
            }
        }

        private void btnChangePassword_Click(object sender, RoutedEventArgs e)
        {
            // Realizamos diversas comprobaciones antes de intentar cambiar la contraseña
            if (this.tbxActualPassword.Password != this.m_Usuario.Contraseña)
            {
                MessageBox.Show("La contraseña actual proporcionada es incorrecta.", "Error", MessageBoxButton.OK);
            }
            else if (string.IsNullOrWhiteSpace(this.tbxPassword1.Password) ||
                     this.tbxPassword1.Password.Length < 5)
            {
                MessageBox.Show("La contraseña que se intenta establecer no cumple con los requerimientos mínimos de seguridad.", "Error", MessageBoxButton.OK);
            }
            else if (this.tbxPassword1.Password != this.tbxPassword2.Password)
            {
                MessageBox.Show("La confirmación de la contraseña no coincide con la contraseña que se intenta establecer.", "Error", MessageBoxButton.OK);
            }
            else if (this.tbxActualPassword.Password == this.tbxPassword1.Password)
            {
                MessageBox.Show("La contraseña que se intenta establecer es idéntica a la actual.", "Error", MessageBoxButton.OK);
            }
            else
            {
                // Llamamos al método del servicio web para cambiar la contraseña
                this.m_InspeccionClient.ChangeUserPasswordAsync(this.m_Usuario, this.tbxPassword1.Password);
            }
        }
    }
}
