﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.Serialization;
using System.ServiceModel;
using System.Text;

namespace InteLect.Services
{
    /// <summary>
    /// Definición del servicio web para InteLect Administración Web
    /// </summary>
    [ServiceContract]
    public interface IInteLectAdministracion
    {
        #region Usuarios

        /// <summary>
        /// Obtiene el listado de usuarios
        /// </summary>
        /// <param name="user">Administrativo</param>
        /// <returns>El listado de usuarios</returns>
        [OperationContract]
        IEnumerable<Usuario> ListUsers(Administrativo user);

        /// <summary>
        /// Obtiene un usuario por su identificador
        /// </summary>
        /// <param name="user">Administrativo</param>
        /// <param name="userId">Identificador del usuario</param>
        /// <returns>El usuario</returns>
        [OperationContract]
        Usuario GetUser(Administrativo user, Guid userId);

        /// <summary>
        /// Autentica un administrativo
        /// </summary>
        /// <param name="username">Nombre de usuario</param>
        /// <param name="password">Contraseña</param>
        /// <returns>El administrativo autenticado</returns>
        [OperationContract]
        Administrativo AuthenticateUser(string username, string password);

        /// <summary>
        /// Cambia la contraseña de un administrativo
        /// </summary>
        /// <param name="user">Administrativo</param>
        /// <param name="newPassword">Nueva contraseña</param>
        /// <returns>Un valor indicando si ha sido posible cambiar la contraseña del administrativo o no</returns>
        [OperationContract]
        bool ChangeUserPassword(Administrativo user, string newPassword);

        #endregion

        #region Compañías

        /// <summary>
        /// Obtiene el listado de compañías
        /// </summary>
        /// <param name="user">Administrativo</param>
        /// <returns>El listado de compañías</returns>
        [OperationContract]
        IEnumerable<Compañia> ListCompanies(Administrativo user);

        /// <summary>
        /// Obtiene una compañía por su identificador
        /// </summary>
        /// <param name="user">Administrativo</param>
        /// <param name="companyId">Identificador de la compañía</param>
        /// <returns>La compañía</returns>
        [OperationContract]
        Compañia GetCompany(Administrativo user, Guid companyId);

        /// <summary>
        /// Actualiza los datos de una compañía
        /// </summary>
        /// <param name="user">Administrativo</param>
        /// <param name="company">Compañía</param>
        /// <returns>Un valor indicando si ha sido posible actualizar la compañía o no</returns>
        [OperationContract]
        bool UpdateCompany(Administrativo user, Compañia company);

        /// <summary>
        /// Crea una nueva compañía
        /// </summary>
        /// <param name="user">Administrativo</param>
        /// <param name="company">Compañía</param>
        /// <returns>Un valor indicando si ha sido posible crear la compañía o no</returns>
        [OperationContract]
        bool AddCompany(Administrativo user, Compañia company);

        /// <summary>
        /// Da de baja una compañía
        /// </summary>
        /// <param name="user">Administrativo</param>
        /// <param name="companyId">Identificador de la compañía</param>
        /// <returns>Un valor indicando si ha sido posible dar de baja la compañía o no</returns>
        [OperationContract]
        bool DeleteCompany(Administrativo user, Guid companyId);

        #endregion

        #region Medidas

        /// <summary>
        /// Obtiene el listado de medidas de una compañía
        /// </summary>
        /// <param name="user">Administrativo</param>
        /// <param name="companyId">Identificador de la compañía</param>
        /// <returns>El listado de medidas de la compañía</returns>
        [OperationContract]
        IEnumerable<Medida> ListMeasures(Administrativo user, Guid companyId);

        /// <summary>
        /// Obtiene una medida por su identificador
        /// </summary>
        /// <param name="user">Administrativo</param>
        /// <param name="measureId">Identificador de la medida</param>
        /// <returns>La medida</returns>
        [OperationContract]
        Medida GetMeasure(Administrativo user, Guid measureId);

        /// <summary>
        /// Actualiza los datos de una medida
        /// </summary>
        /// <param name="user">Administrativo</param>
        /// <param name="measure">Medida</param>
        /// <returns>Un valor indicando si ha sido posible actualizar la medida o no</returns>
        [OperationContract]
        bool UpdateMeasure(Administrativo user, Medida measure);

        /// <summary>
        /// Crea una nueva medida
        /// </summary>
        /// <param name="user">Administrativo</param>
        /// <param name="companyId">Identificador de la compañía</param>
        /// <param name="measure">Medida</param>
        /// <returns>Un valor indicando si ha sido posible crear la medida o no</returns>
        [OperationContract]
        bool AddMeasure(Administrativo user, Guid companyId, Medida measure);

        /// <summary>
        /// Da de baja una medida
        /// </summary>
        /// <param name="user">Administrativo</param>
        /// <param name="measureId">Identificador de la medida</param>
        /// <returns>Un valor indicando si ha sido posible dar de baja la medida o no</returns>
        [OperationContract]
        bool DeleteMeasure(Administrativo user, Guid measureId);

        #endregion

        #region Clientes

        /// <summary>
        /// Obtiene el listado de clientes
        /// </summary>
        /// <param name="user">Administrativo</param>
        /// <returns>El listado de clientes</returns>
        [OperationContract]
        IEnumerable<Cliente> ListClients(Administrativo user);

        /// <summary>
        /// Obtiene un cliente por su identificador
        /// </summary>
        /// <param name="user">Administrativo</param>
        /// <param name="clientId">Identificador del cliente</param>
        /// <returns>El cliente</returns>
        [OperationContract]
        Cliente GetClient(Administrativo user, Guid clientId);

        /// <summary>
        /// Actualiza los datos de un cliente
        /// </summary>
        /// <param name="user">Administrativo</param>
        /// <param name="client">Cliente</param>
        /// <returns>Un valor indicando si ha sido posible actualizar el cliente o no</returns>
        [OperationContract]
        bool UpdateClient(Administrativo user, Cliente client);

        /// <summary>
        /// Crea un nuevo cliente
        /// </summary>
        /// <param name="user">Administrativo</param>
        /// <param name="client">Cliente</param>
        /// <returns>Un valor indicando si ha sido posible crear el cliente o no</returns>
        [OperationContract]
        bool AddClient(Administrativo user, Cliente client);

        /// <summary>
        /// Da de baja un cliente
        /// </summary>
        /// <param name="user">Administrativo</param>
        /// <param name="clientId">Identificador del cliente</param>
        /// <returns>Un valor indicando si ha sido posible dar de baja el cliente o no</returns>
        [OperationContract]
        bool DeleteClient(Administrativo user, Guid clientId);

        #endregion

        #region Direcciones

        /// <summary>
        /// Obtiene el listado de direcciones de un cliente
        /// </summary>
        /// <param name="user">Administrativo</param>
        /// <param name="clientId">Identificador del cliente</param>
        /// <returns>El listado de direcciones del cliente</returns>
        [OperationContract]
        IEnumerable<Direccion> ListAddresses(Administrativo user, Guid clientId);

        /// <summary>
        /// Obtiene una dirección por su identificador
        /// </summary>
        /// <param name="user">Administrativo</param>
        /// <param name="addressId">Identificador de la dirección</param>
        /// <returns>La dirección</returns>
        [OperationContract]
        Direccion GetAddress(Administrativo user, Guid addressId);

        /// <summary>
        /// Actualiza los datos de una dirección
        /// </summary>
        /// <param name="user">Administrativo</param>
        /// <param name="address">Dirección</param>
        /// <returns>Un valor indicando si ha sido posible actualizar la dirección o no</returns>
        [OperationContract]
        bool UpdateAddress(Administrativo user, Direccion address);

        /// <summary>
        /// Crea una nueva dirección
        /// </summary>
        /// <param name="user">Administrativo</param>
        /// <param name="clientId">Identificador del cliente</param>
        /// <param name="address">Dirección</param>
        /// <returns>Un valor indicando si ha sido posible crear la dirección o no</returns>
        [OperationContract]
        bool AddAddress(Administrativo user, Guid clientId, Direccion address);

        /// <summary>
        /// Da de baja una dirección
        /// </summary>
        /// <param name="user">Administrativo</param>
        /// <param name="addressId">Identificador de la dirección</param>
        /// <returns>Un valor indicando si ha sido posible dar de baja la dirección o no</returns>
        [OperationContract]
        bool DeleteAddress(Administrativo user, Guid addressId);

        #endregion

        #region Servicios

        /// <summary>
        /// Obtiene el listado de servicios
        /// </summary>
        /// <param name="user">Administrativo</param>
        /// <returns>El listado de servicios</returns>
        [OperationContract]
        IEnumerable<Servicio> ListServices(Administrativo user);

        /// <summary>
        /// Obtiene un servicio por su identificador
        /// </summary>
        /// <param name="user">Administrativo</param>
        /// <param name="serviceId">Identificador del servicio</param>
        /// <returns>El servicio</returns>
        [OperationContract]
        Servicio GetService(Administrativo user, Guid serviceId);

        /// <summary>
        /// Actualiza los datos de un servicio
        /// </summary>
        /// <param name="user">Administrativo</param>
        /// <param name="service">Servicio</param>
        /// <returns>Un valor indicando si ha sido posible actualizar el servicio o no</returns>
        [OperationContract]
        bool UpdateService(Administrativo user, Servicio service);

        /// <summary>
        /// Crea un nuevo servicio
        /// </summary>
        /// <param name="user">Administrativo</param>
        /// <param name="measureId">Identificador de la medida</param>
        /// <param name="addressId">Identificador de la dirección</param>
        /// <param name="service">Servicio</param>
        /// <returns>Un valor indicando si ha sido posible crear el servicio o no</returns>
        [OperationContract]
        bool AddService(Administrativo user, Guid measureId, Guid addressId, Servicio service);

        /// <summary>
        /// Da de baja un servicio
        /// </summary>
        /// <param name="user">Administrativo</param>
        /// <param name="serviceId">Identificador del servicio</param>
        /// <returns>Un valor indicando si ha sido posible dar de baja el servicio o no</returns>
        [OperationContract]
        bool DeleteService(Administrativo user, Guid serviceId);

        #endregion

        #region Órdenes de trabajo

        /// <summary>
        /// Obtiene el listado de órdenes de trabajo
        /// </summary>
        /// <param name="user">Administrativo</param>
        /// <returns>El listado de órdenes de trabajo</returns>
        [OperationContract]
        IEnumerable<OrdenTrabajo> ListWorkOrders(Administrativo user);

        /// <summary>
        /// Obtiene una órden de trabajo por su identificador
        /// </summary>
        /// <param name="user">Administrativo</param>
        /// <param name="workOrderId">Identificador de la órden de trabajo</param>
        /// <returns>La órden de trabajo</returns>
        [OperationContract]
        OrdenTrabajo GetWorkOrder(Administrativo user, Guid workOrderId);

        /// <summary>
        /// Actualiza los datos de una órden de trabajo
        /// </summary>
        /// <param name="user">Administrativo</param>
        /// <param name="workOrder">Órden de trabajo</param>
        /// <returns>Un valor indicando si ha sido posible actualizar la órden de trabajo o no</returns>
        [OperationContract]
        bool UpdateWorkOrder(Administrativo user, OrdenTrabajo workOrder);

        /// <summary>
        /// Crea una nueva órden de trabajo
        /// </summary>
        /// <param name="user">Administrativo</param>
        /// <param name="inspectorId">Identificador del inspector</param>
        /// <param name="serviceId">Identificador del servicio</param>
        /// <param name="workOrder">Órden de trabajo</param>
        /// <returns>Un valor indicando si ha sido posible crear la órden de trabajo o no</returns>
        [OperationContract]
        bool AddWorkOrder(Administrativo user, Guid inspectorId, Guid serviceId, OrdenTrabajo workOrder);

        /// <summary>
        /// Elimina una órden de trabajo
        /// </summary>
        /// <param name="user">Administrativo</param>
        /// <param name="workOrderId">Identificador de la órden de trabajo</param>
        /// <returns>Un valor indicando si ha sido posible dar de baja la órden de trabajo o no</returns>
        [OperationContract]
        bool DeleteWorkOrder(Administrativo user, Guid workOrderId);

        #endregion

    }
}
