﻿using System.Collections.Generic;
using System.ServiceModel;

namespace InteLect.Services
{
    /// <summary>
    /// Definición del servicio web para InteLect Inspección Móvil
    /// </summary>
    [ServiceContract]
    public interface IInteLectInspeccion
    {
        /// <summary>
        /// Autentica un inspector
        /// </summary>
        /// <param name="username">Nombre de usuario</param>
        /// <param name="password">Contraseña</param>
        /// <returns>El inspector autenticado</returns>
        [OperationContract]
        UsuarioMovil AuthenticateUser(string username, string password);

        /// <summary>
        /// Cambia la contraseña de un inspector
        /// </summary>
        /// <param name="user">Inspector</param>
        /// <param name="newPassword">Nueva contraseña</param>
        /// <returns>Un valor indicando si ha sido posible cambiar la contraseña del inspector o no</returns>
        [OperationContract]
        bool ChangeUserPassword(UsuarioMovil user, string newPassword);

        /// <summary>
        /// Descarga el plan de trabajo de un inspector
        /// </summary>
        /// <param name="user">Inspector</param>
        /// <returns>El plan de trabajo del inspector</returns>
        [OperationContract]
        IEnumerable<OrdenTrabajoMovil> DownloadWorkPlan(UsuarioMovil user);

        /// <summary>
        /// Actualiza una orden de trabajo
        /// </summary>
        /// <param name="user">Inspector</param>
        /// <param name="workOrder">Orden de trabajo</param>
        /// <returns>Un valor indicando si ha sido posible actualizar la orden de trabajo o no</returns>
        [OperationContract]
        bool UpdateWorkOrder(UsuarioMovil user, OrdenTrabajoMovil workOrder);
    }
}
