﻿using System;
using System.Linq;
using System.Web.Mvc;
using InteLect.Web.InteLectAdministracionService;

namespace InteLect.Web.Controllers
{
    public class ClientesController : Controller
    {
        private InteLectAdministracionClient m_AdministracionClient = new InteLectAdministracionClient(); // Acceso al servicio web de administración
        private Administrativo m_Usuario = System.Web.HttpContext.Current.Session["User"] as Administrativo; // Usuario actual

        public ActionResult Index()
        {
            return this.View(this.m_AdministracionClient.ListClients(this.m_Usuario).OrderBy(entity => entity.Apellidos));
        }

        public ActionResult Detalles(Guid id)
        {
            return this.View(this.m_AdministracionClient.GetClient(this.m_Usuario, id));
        }

        public ActionResult Editar(Guid id)
        {
            return this.View(this.m_AdministracionClient.GetClient(this.m_Usuario, id));
        }

        [HttpPost]
        public ActionResult Guardar(Cliente client)
        {
            if (string.IsNullOrWhiteSpace(client.Codigo))
            {
                this.ModelState.AddModelError("Codigo", "El código proporcionado no es válido");
            }

            if (string.IsNullOrWhiteSpace(client.Nombre))
            {
                this.ModelState.AddModelError("Nombre", "El nombre proporcionado no es válido");
            }

            if (string.IsNullOrWhiteSpace(client.Nombre))
            {
                this.ModelState.AddModelError("Apellidos", "Los apellidos proporcionados no son válidos");
            }

            if (!this.ModelState.IsValid)
            {
                return this.View("Editar", client);
            }

            if (this.m_AdministracionClient.UpdateClient(this.m_Usuario, client))
            {
                return this.View("Index", this.m_AdministracionClient.ListClients(this.m_Usuario).OrderBy(entity => entity.Apellidos));
            }
            else
            {
                throw new Exception("No se han podido guardar los datos del cliente.");
            }
        }

        public ActionResult Nuevo()
        {
            return this.View(new Cliente());
        }

        [HttpPost]
        public ActionResult GuardarNuevo(Cliente client)
        {
            if (string.IsNullOrWhiteSpace(client.Codigo))
            {
                this.ModelState.AddModelError("Codigo", "El código proporcionado no es válido");
            }

            if (string.IsNullOrWhiteSpace(client.Nombre))
            {
                this.ModelState.AddModelError("Nombre", "El nombre proporcionado no es válido");
            }

            if (string.IsNullOrWhiteSpace(client.Nombre))
            {
                this.ModelState.AddModelError("Apellidos", "Los apellidos proporcionados no son válidos");
            }

            if (!this.ModelState.IsValid)
            {
                return this.View("Nuevo", client);
            }

            if (this.m_AdministracionClient.AddClient(this.m_Usuario, client))
            {
                return this.View("Index", this.m_AdministracionClient.ListClients(this.m_Usuario).OrderBy(entity => entity.Apellidos));
            }
            else
            {
                throw new Exception("No se ha podido crear el cliente.");
            }
        }

        public ActionResult Baja(Guid id)
        {
            if (this.m_AdministracionClient.DeleteClient(this.m_Usuario, id))
            {
                return this.View("Index", this.m_AdministracionClient.ListClients(this.m_Usuario).OrderBy(entity => entity.Apellidos));
            }
            else
            {
                throw new Exception("No se ha podido dar de baja el cliente.");
            }
        }
    }
}
