﻿using System;
using System.Linq;
using System.Web.Mvc;
using InteLect.Web.InteLectAdministracionService;

namespace InteLect.Web.Controllers
{
    public class CompañiasController : Controller
    {
        private InteLectAdministracionClient m_AdministracionClient = new InteLectAdministracionClient(); // Acceso al servicio web de administración
        private Administrativo m_Usuario = System.Web.HttpContext.Current.Session["User"] as Administrativo; // Usuario actual

        public ActionResult Index()
        {
            return this.View(this.m_AdministracionClient.ListCompanies(this.m_Usuario).OrderBy(entity => entity.Nombre));
        }

        public ActionResult Detalles(Guid id)
        {
            return this.View(this.m_AdministracionClient.GetCompany(this.m_Usuario, id));
        }

        public ActionResult Editar(Guid id)
        {
            return this.View(this.m_AdministracionClient.GetCompany(this.m_Usuario, id));
        }

        [HttpPost]
        public ActionResult Guardar(Compañia company)
        {
            if (string.IsNullOrWhiteSpace(company.Codigo))
            {
                this.ModelState.AddModelError("Codigo", "El código proporcionado no es válido");
            }

            if (string.IsNullOrWhiteSpace(company.Nombre))
            {
                this.ModelState.AddModelError("Nombre", "El nombre proporcionado no es válido");
            }

            if (!this.ModelState.IsValid)
            {
                return this.View("Editar", company);
            }

            if (this.m_AdministracionClient.UpdateCompany(this.m_Usuario, company))
            {
                return this.View("Index", this.m_AdministracionClient.ListCompanies(this.m_Usuario).OrderBy(entity => entity.Nombre));
            }
            else
            {
                throw new Exception("No se han podido guardar los datos de la compañía.");
            }
        }

        public ActionResult Nueva()
        {
            return this.View(new Compañia());
        }

        [HttpPost]
        public ActionResult GuardarNueva(Compañia company)
        {
            if (string.IsNullOrWhiteSpace(company.Codigo))
            {
                this.ModelState.AddModelError("Codigo", "El código proporcionado no es válido");
            }

            if (string.IsNullOrWhiteSpace(company.Nombre))
            {
                this.ModelState.AddModelError("Nombre", "El nombre proporcionado no es válido");
            }

            if (!this.ModelState.IsValid)
            {
                return this.View("Nueva", company);
            }

            if (this.m_AdministracionClient.AddCompany(this.m_Usuario, company))
            {
                return this.View("Index", this.m_AdministracionClient.ListCompanies(this.m_Usuario).OrderBy(entity => entity.Nombre));
            }
            else
            {
                throw new Exception("No se ha podido crear la compañía.");
            }
        }

        public ActionResult Baja(Guid id)
        {
            if (this.m_AdministracionClient.DeleteCompany(this.m_Usuario, id))
            {
                return this.View("Index", this.m_AdministracionClient.ListCompanies(this.m_Usuario).OrderBy(entity => entity.Nombre));
            }
            else
            {
                throw new Exception("No se ha podido dar de baja la compañía.");
            }
        }
    }
}
