﻿using System;
using System.Linq;
using System.Web.Mvc;
using InteLect.Web.InteLectAdministracionService;

namespace InteLect.Web.Controllers
{
    public class DireccionesController : Controller
    {
        private InteLectAdministracionClient m_AdministracionClient = new InteLectAdministracionClient(); // Acceso al servicio web de administración
        private Administrativo m_Usuario = System.Web.HttpContext.Current.Session["User"] as Administrativo; // Usuario actual

        public ActionResult Index(Guid id)
        {
            this.ViewData["Client"] = this.m_AdministracionClient.GetClient(this.m_Usuario, id);
            return View(this.m_AdministracionClient.ListAddresses(this.m_Usuario, id).OrderBy(entity => entity.Nombre));
        }

        public ActionResult Detalles(Guid id)
        {
            var address = this.m_AdministracionClient.GetAddress(this.m_Usuario, id);
            this.ViewData["Client"] = this.m_AdministracionClient.GetClient(this.m_Usuario, address.ClienteId);
            return this.View(address);
        }

        public ActionResult Editar(Guid id)
        {
            var address = this.m_AdministracionClient.GetAddress(this.m_Usuario, id);
            this.ViewData["Client"] = this.m_AdministracionClient.GetClient(this.m_Usuario, address.ClienteId);
            return this.View(address);
        }

        [HttpPost]
        public ActionResult Guardar(Direccion address)
        {
            this.ViewData["Client"] = this.m_AdministracionClient.GetClient(this.m_Usuario, address.ClienteId);

            if (string.IsNullOrWhiteSpace(address.Codigo))
            {
                this.ModelState.AddModelError("Codigo", "El código proporcionado no es válido");
            }

            if (string.IsNullOrWhiteSpace(address.Nombre))
            {
                this.ModelState.AddModelError("Nombre", "El nombre proporcionado no es válido");
            }

            if (string.IsNullOrWhiteSpace(address.CodigoPostal))
            {
                this.ModelState.AddModelError("CodigoPostal", "El código postal proporcionado no es válido");
            }

            if (string.IsNullOrWhiteSpace(address.Poblacion))
            {
                this.ModelState.AddModelError("Poblacion", "La población proporcionada no es válida");
            }

            if (string.IsNullOrWhiteSpace(address.Provincia))
            {
                this.ModelState.AddModelError("Provincia", "La provincia proporcionada no es válida");
            }

            if (!this.ModelState.IsValid)
            {
                return this.View("Editar", address);
            }

            if (this.m_AdministracionClient.UpdateAddress(this.m_Usuario, address))
            {
                return this.View("Index", this.m_AdministracionClient.ListAddresses(this.m_Usuario, address.ClienteId).OrderBy(entity => entity.Nombre));
            }
            else
            {
                throw new Exception("No se han podido guardar los datos de la dirección.");
            }
        }

        public ActionResult Nueva(Guid id)
        {
            this.ViewData["Client"] = this.m_AdministracionClient.GetClient(this.m_Usuario, id);
            return this.View(new Direccion() { ClienteId = id });
        }

        [HttpPost]
        public ActionResult GuardarNueva(Direccion address)
        {
            this.ViewData["Client"] = this.m_AdministracionClient.GetClient(this.m_Usuario, address.ClienteId);

            if (string.IsNullOrWhiteSpace(address.Codigo))
            {
                this.ModelState.AddModelError("Codigo", "El código proporcionado no es válido");
            }

            if (string.IsNullOrWhiteSpace(address.Nombre))
            {
                this.ModelState.AddModelError("Nombre", "El nombre proporcionado no es válido");
            }

            if (string.IsNullOrWhiteSpace(address.CodigoPostal))
            {
                this.ModelState.AddModelError("CodigoPostal", "El código postal proporcionado no es válido");
            }

            if (string.IsNullOrWhiteSpace(address.Poblacion))
            {
                this.ModelState.AddModelError("Poblacion", "La población proporcionada no es válida");
            }

            if (string.IsNullOrWhiteSpace(address.Provincia))
            {
                this.ModelState.AddModelError("Provincia", "La provincia proporcionada no es válida");
            }

            if (!this.ModelState.IsValid)
            {
                return this.View("Nueva", address);
            }

            if (this.m_AdministracionClient.AddAddress(this.m_Usuario, address.ClienteId, address))
            {
                return this.View("Index", this.m_AdministracionClient.ListAddresses(this.m_Usuario, address.ClienteId).OrderBy(entity => entity.Nombre));
            }
            else
            {
                throw new Exception("No se ha podido crear la dirección.");
            }
        }

        public ActionResult Baja(Guid id)
        {
            var address = this.m_AdministracionClient.GetAddress(this.m_Usuario, id);
            this.ViewData["Client"] = this.m_AdministracionClient.GetClient(this.m_Usuario, address.ClienteId);

            if (this.m_AdministracionClient.DeleteAddress(this.m_Usuario, id))
            {
                return this.View("Index", this.m_AdministracionClient.ListAddresses(this.m_Usuario, address.ClienteId).OrderBy(entity => entity.Nombre));
            }
            else
            {
                throw new Exception("No se ha podido dar de baja la dirección.");
            }
        }
    }
}
