﻿using System;
using System.Linq;
using System.Web.Mvc;
using InteLect.Web.InteLectAdministracionService;

namespace InteLect.Web.Controllers
{
    public class MedidasController : Controller
    {
        private InteLectAdministracionClient m_AdministracionClient = new InteLectAdministracionClient(); // Acceso al servicio web de administración
        private Administrativo m_Usuario = System.Web.HttpContext.Current.Session["User"] as Administrativo; // Usuario actual

        public ActionResult Index(Guid id)
        {
            this.ViewData["Company"] = this.m_AdministracionClient.GetCompany(this.m_Usuario, id);
            return View(this.m_AdministracionClient.ListMeasures(this.m_Usuario, id).OrderBy(entity => entity.Nombre));
        }

        public ActionResult Detalles(Guid id)
        {
            var medida = this.m_AdministracionClient.GetMeasure(this.m_Usuario, id);
            this.ViewData["Company"] = this.m_AdministracionClient.GetCompany(this.m_Usuario, medida.CompañiaId);
            return this.View(medida);
        }

        public ActionResult Editar(Guid id)
        {
            var medida = this.m_AdministracionClient.GetMeasure(this.m_Usuario, id);
            this.ViewData["Company"] = this.m_AdministracionClient.GetCompany(this.m_Usuario, medida.CompañiaId);
            return this.View(medida);
        }

        [HttpPost]
        public ActionResult Guardar(Medida measure)
        {
            this.ViewData["Company"] = this.m_AdministracionClient.GetCompany(this.m_Usuario, measure.CompañiaId);

            if (string.IsNullOrWhiteSpace(measure.Codigo))
            {
                this.ModelState.AddModelError("Codigo", "El código proporcionado no es válido");
            }

            if (string.IsNullOrWhiteSpace(measure.Nombre))
            {
                this.ModelState.AddModelError("Nombre", "El nombre proporcionado no es válido");
            }

            if (string.IsNullOrWhiteSpace(measure.Unidades))
            {
                this.ModelState.AddModelError("Unidades", "Las unidades proporcionadas no son válidas");
            }

            if (!this.ModelState.IsValid)
            {
                return this.View("Editar", measure);
            }

            if (this.m_AdministracionClient.UpdateMeasure(this.m_Usuario, measure))
            {
                return this.View("Index", this.m_AdministracionClient.ListMeasures(this.m_Usuario, measure.CompañiaId).OrderBy(entity => entity.Nombre));
            }
            else
            {
                throw new Exception("No se han podido guardar los datos de la medida.");
            }
        }

        public ActionResult Nueva(Guid id)
        {
            this.ViewData["Company"] = this.m_AdministracionClient.GetCompany(this.m_Usuario, id);
            return this.View(new Medida() { CompañiaId = id });
        }

        [HttpPost]
        public ActionResult GuardarNueva(Medida measure)
        {
            this.ViewData["Company"] = this.m_AdministracionClient.GetCompany(this.m_Usuario, measure.CompañiaId);

            if (string.IsNullOrWhiteSpace(measure.Codigo))
            {
                this.ModelState.AddModelError("Codigo", "El código proporcionado no es válido");
            }

            if (string.IsNullOrWhiteSpace(measure.Nombre))
            {
                this.ModelState.AddModelError("Nombre", "El nombre proporcionado no es válido");
            }

            if (string.IsNullOrWhiteSpace(measure.Unidades))
            {
                this.ModelState.AddModelError("Unidades", "Las unidades proporcionadas no son válidas");
            }

            if (!this.ModelState.IsValid)
            {
                return this.View("Nueva", measure);
            }

            if (this.m_AdministracionClient.AddMeasure(this.m_Usuario, measure.CompañiaId, measure))
            {
                return this.View("Index", this.m_AdministracionClient.ListMeasures(this.m_Usuario, measure.CompañiaId).OrderBy(entity => entity.Nombre));
            }
            else
            {
                throw new Exception("No se ha podido crear la medida.");
            }
        }

        public ActionResult Baja(Guid id)
        {
            var medida = this.m_AdministracionClient.GetMeasure(this.m_Usuario, id);
            this.ViewData["Company"] = this.m_AdministracionClient.GetCompany(this.m_Usuario, medida.CompañiaId);

            if (this.m_AdministracionClient.DeleteMeasure(this.m_Usuario, id))
            {
                return this.View("Index", this.m_AdministracionClient.ListMeasures(this.m_Usuario, medida.CompañiaId).OrderBy(entity => entity.Nombre));
            }
            else
            {
                throw new Exception("No se ha podido dar de baja la medida.");
            }
        }
    }
}
