﻿using System;
using System.Linq;
using System.Web.Mvc;
using InteLect.Web.InteLectAdministracionService;

namespace InteLect.Web.Controllers
{
    public class OrdenesTrabajoController : Controller
    {
        private InteLectAdministracionClient m_AdministracionClient = new InteLectAdministracionClient(); // Acceso al servicio web de administración
        private Administrativo m_Usuario = System.Web.HttpContext.Current.Session["User"] as Administrativo; // Usuario actual

        public ActionResult Index()
        {
            return this.View(this.m_AdministracionClient.ListWorkOrders(this.m_Usuario).OrderBy(entity => entity.Fecha));
        }

        public ActionResult Detalles(Guid id)
        {
            return this.View(this.m_AdministracionClient.GetWorkOrder(this.m_Usuario, id));
        }

        public ActionResult Editar(Guid id)
        {
            return this.View(this.m_AdministracionClient.GetWorkOrder(this.m_Usuario, id));
        }

        [HttpPost]
        public ActionResult Guardar(OrdenTrabajo workOrder)
        {
            if (!this.ModelState.IsValid)
            {
                return this.View("Editar", workOrder);
            }

            if (this.m_AdministracionClient.UpdateWorkOrder(this.m_Usuario, workOrder))
            {
                return this.View("Index", this.m_AdministracionClient.ListWorkOrders(this.m_Usuario).OrderBy(entity => entity.Fecha));
            }
            else
            {
                throw new Exception("No se han podido guardar los datos de la órden de trabajo.");
            }
        }

        public ActionResult Nueva()
        {
            return this.View(new OrdenTrabajo() { Fecha = DateTime.Today });
        }

        [HttpPost]
        public ActionResult GuardarNueva(OrdenTrabajo workOrder)
        {
            if (workOrder.ServicioId == Guid.Empty)
            {
                this.ModelState.AddModelError("ServicioId", "Selecciona un servicio");
            }

            if (workOrder.UsuarioId == Guid.Empty)
            {
                this.ModelState.AddModelError("UsuarioId", "Selecciona un inspector");
            }

            if (!this.ModelState.IsValid)
            {
                return this.View("Nueva", workOrder);
            }

            if (this.m_AdministracionClient.AddWorkOrder(this.m_Usuario, workOrder.UsuarioId, workOrder.ServicioId, workOrder))
            {
                return this.View("Index", this.m_AdministracionClient.ListWorkOrders(this.m_Usuario).OrderBy(entity => entity.Fecha));
            }
            else
            {
                throw new Exception("No se ha podido crear la órden de trabajo.");
            }
        }

        public ActionResult Eliminar(Guid id)
        {
            if (this.m_AdministracionClient.DeleteWorkOrder(this.m_Usuario, id))
            {
                return this.View("Index", this.m_AdministracionClient.ListWorkOrders(this.m_Usuario).OrderBy(entity => entity.Fecha));
            }
            else
            {
                throw new Exception("No se ha podido eliminar la órden de trabajo.");
            }
        }

        #region Ajax

        [HttpPost]
        public ActionResult GetClientByServiceId(Guid id)
        {
            var servicio = this.m_AdministracionClient.GetService(this.m_Usuario, id);
            return this.Json(this.m_AdministracionClient.GetClient(this.m_Usuario, this.m_AdministracionClient.GetAddress(this.m_Usuario, servicio.DireccionId).ClienteId));
        }

        [HttpPost]
        public ActionResult GetAddressByServiceId(Guid id)
        {
            var servicio = this.m_AdministracionClient.GetService(this.m_Usuario, id);
            return this.Json(this.m_AdministracionClient.GetAddress(this.m_Usuario, servicio.DireccionId));
        }

        [HttpPost]
        public ActionResult GetCompanyByServiceId(Guid id)
        {
            var servicio = this.m_AdministracionClient.GetService(this.m_Usuario, id);
            return this.Json(this.m_AdministracionClient.GetCompany(this.m_Usuario, this.m_AdministracionClient.GetMeasure(this.m_Usuario, servicio.MedidaId).CompañiaId));
        }

        [HttpPost]
        public ActionResult GetMeasureByServiceId(Guid id)
        {
            var servicio = this.m_AdministracionClient.GetService(this.m_Usuario, id);
            return this.Json(this.m_AdministracionClient.GetMeasure(this.m_Usuario, servicio.MedidaId));
        }

        [HttpPost]
        public ActionResult GetInspectorById(Guid id)
        {
            return this.Json(this.m_AdministracionClient.GetUser(this.m_Usuario, id));
        }

        [HttpPost]
        public ActionResult GetInspectors()
        {
            return this.Json(this.m_AdministracionClient.ListUsers(this.m_Usuario).Union(new[] { new Inspector() { UsuarioId = Guid.Empty, Tipo = "INS", Nombre = string.Empty, Apellidos = string.Empty } }).OfType<Inspector>().Where(entity => entity.FechaBaja == null).OrderBy(entity => entity.Apellidos));
        }

        [HttpPost]
        public ActionResult GetServices()
        {
            return this.Json(this.m_AdministracionClient.ListServices(this.m_Usuario).Union(new[] { new Servicio() { ServicioId = Guid.Empty, Codigo = string.Empty } }).OrderBy(entity => entity.Codigo));
        }

        #endregion
    }
}
