﻿using System;
using System.Linq;
using System.Web.Mvc;
using InteLect.Web.InteLectAdministracionService;

namespace InteLect.Web.Controllers
{
    public class ServiciosController : Controller
    {
        private InteLectAdministracionClient m_AdministracionClient = new InteLectAdministracionClient(); // Acceso al servicio web de administración
        private Administrativo m_Usuario = System.Web.HttpContext.Current.Session["User"] as Administrativo; // Usuario actual

        public ActionResult Index()
        {
            return View(this.m_AdministracionClient.ListServices(this.m_Usuario).OrderBy(entity => entity.Codigo));
        }

        public ActionResult Detalles(Guid id)
        {
            return this.View(this.m_AdministracionClient.GetService(this.m_Usuario, id));
        }

        public ActionResult Editar(Guid id)
        {
            return this.View(this.m_AdministracionClient.GetService(this.m_Usuario, id));
        }

        [HttpPost]
        public ActionResult Guardar(Servicio service)
        {
            if (string.IsNullOrWhiteSpace(service.Codigo))
            {
                this.ModelState.AddModelError("Codigo", "El código proporcionado no es válido");
            }

            if (!this.ModelState.IsValid)
            {
                return this.View("Editar", service);
            }

            if (this.m_AdministracionClient.UpdateService(this.m_Usuario, service))
            {
                return this.View("Index", this.m_AdministracionClient.ListServices(this.m_Usuario).OrderBy(entity => entity.Codigo));
            }
            else
            {
                throw new Exception("No se han podido guardar los datos del servicio.");
            }
        }

        public ActionResult Nuevo()
        {
            return this.View(new Servicio());
        }

        [HttpPost]
        public ActionResult GuardarNuevo(Servicio service)
        {
            if (string.IsNullOrWhiteSpace(service.Codigo))
            {
                this.ModelState.AddModelError("Codigo", "El código proporcionado no es válido");
            }

            if (service.DireccionId == Guid.Empty)
            {
                this.ModelState.AddModelError("DireccionId", "Selecciona una dirección");
            }

            if (service.MedidaId == Guid.Empty)
            {
                this.ModelState.AddModelError("MedidaId", "Selecciona una medida");
            }

            if (!this.ModelState.IsValid)
            {
                return this.View("Nuevo", service);
            }

            if (this.m_AdministracionClient.AddService(this.m_Usuario, service.MedidaId, service.DireccionId, service))
            {
                return this.View("Index", this.m_AdministracionClient.ListServices(this.m_Usuario).OrderBy(entity => entity.Codigo));
            }
            else
            {
                throw new Exception("No se ha podido crear el servicio.");
            }
        }

        public ActionResult Baja(Guid id)
        {
            if (this.m_AdministracionClient.DeleteService(this.m_Usuario, id))
            {
                return this.View("Index", this.m_AdministracionClient.ListServices(this.m_Usuario).OrderBy(entity => entity.Codigo));
            }
            else
            {
                throw new Exception("No se ha podido dar de baja el servicio.");
            }
        }

        #region Ajax
        
        [HttpPost]
        public ActionResult GetClientByAddressId(Guid id)
        {
            return this.Json(this.m_AdministracionClient.GetClient(this.m_Usuario, this.m_AdministracionClient.GetAddress(this.m_Usuario, id).ClienteId));
        }

        [HttpPost]
        public ActionResult GetAddressById(Guid id)
        {
            return this.Json(this.m_AdministracionClient.GetAddress(this.m_Usuario, id));
        }

        [HttpPost]
        public ActionResult GetCompanyByMeasureId(Guid id)
        {
            return this.Json(this.m_AdministracionClient.GetCompany(this.m_Usuario, this.m_AdministracionClient.GetMeasure(this.m_Usuario, id).CompañiaId));
        }

        [HttpPost]
        public ActionResult GetMeasureById(Guid id)
        {
            return this.Json(this.m_AdministracionClient.GetMeasure(this.m_Usuario, id));
        }

        [HttpPost]
        public ActionResult GetClients()
        {
            return this.Json(this.m_AdministracionClient.ListClients(this.m_Usuario).Union(new[] { new Cliente() { ClienteId = Guid.Empty, Nombre = string.Empty, Apellidos = string.Empty } }).OrderBy(entity => entity.Apellidos));
        }

        [HttpPost]
        public ActionResult GetAddresesByClientId(Guid id)
        {
            return this.Json(this.m_AdministracionClient.ListAddresses(this.m_Usuario, id).OrderBy(entity => entity.Nombre));
        }

        [HttpPost]
        public ActionResult GetCompanies()
        {
            return this.Json(this.m_AdministracionClient.ListCompanies(this.m_Usuario).Union(new[] { new Compañia() { CompañiaId = Guid.Empty, Nombre = string.Empty } }).OrderBy(entity => entity.Nombre));
        }

        [HttpPost]
        public ActionResult GetMeasuresByCompanyId(Guid id)
        {
            return this.Json(this.m_AdministracionClient.ListMeasures(this.m_Usuario, id).OrderBy(entity => entity.Nombre));
        }

        #endregion
    }
}
